/***************************************************************************
                          tree.c  -  description
                             -------------------
    begin                : Tue Jun 13 2000
    copyright            : (C) 2000 by Olivier ELEMENTO
    email                : olivier@ligm.igh.cnrs.fr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
/***************************************************************************
                          tree.c  -  description
                             -------------------
    begin                : Tue Apr 4 2000
    copyright            : (C) 2000 by olivier ELEMENTO
    email                : elemento@club-internet.fr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "tree.h"
#include <stdio.h>
#include <assert.h>
#include <limits.h>
#include <string.h>
#include <math.h>

#define MAXNUMNODES 50000

/*#define malloc MemTrace_Malloc
#define calloc MemTrace_Calloc
#define free MemTrace_Free
#include "memtrace.h"*/

tree* allocTree(int T) {
	int i;

	
	tree* t = (tree*)malloc(sizeof(tree));
	//t->nodes = (int**)malloc(2 * sizeof(int*));
	//for (i=0; i<2; i++) {
	assert(T>0);
	t->nodes = (int*)malloc((2*T-1) * sizeof(int));
	//t->adjnodes = (int*)malloc(T * sizeof(int));
	t->children = (int**)malloc((2*T-1) * sizeof(int*));
  for (i=0; i<(2*T-1); i++)
    t->children[i] = (int*)malloc(2*sizeof(int));
	t->maxnodes = 2*T-1;
	t->maxadjnodes = T;
	t->maxleaves = T;
	t->maxintnodes = T-1;
	
	return t;
}

void destroy(tree* t) {
	int i;

	free(t->nodes);
	//free(t->adjnodes);
	for (i=0; i<t->maxnodes; i++)
		free(t->children[i]);
    //t->children[i] = (int*)malloc(2*sizeof(int));
  free(t->children);
}
void initTree(tree* t, int T) {
	int i;
	//t->size = 0;
	for (i=0; i<t->maxnodes; i++)
		t->nodes[i] = NOTHING;
	//for (i=0; i<t->maxadjnodes; i++)
	//	t->adjnodes[i] = NOTHING;
	for (i=0; i<t->maxnodes; i++) {
    t->children[i][0] = NOTHING;
    t->children[i][1] = NOTHING;
  }
	t->numnodes = 1;

	t->numadjnodes = 1;
	//t->adjnodes[0] = 0; //premiere seq correspond a la racine

	t->numintnodes = 1;
	t->numleaves = 2;
	t->nodes[t->maxintnodes] = 0;
	t->nodes[t->maxintnodes+1] = 0;
	t->children[0][0] = t->maxintnodes;
	t->children[0][1] = t->maxintnodes+1;
}

int divide(tree* t, int numnode) {
	//les enfants sont forcement t->numnode et t->numnode+1
	int tmp = t->numnodes;
	
	//printf("Before divide\n");
	//showNodes(t);	
	
	t->children[numnode][0] = t->numnodes;
  t->children[numnode][1] = t->numnodes+1;
	t->nodes[t->numnodes] = numnode;
	t->nodes[t->numnodes+1] = numnode;
	t->numnodes = t->numnodes + 2;
	//printf("After divide\n");
	//showNodes(t);
	return tmp;

}


tree* getCopyOfTree(tree* t) {
	int i;
	
	tree* nt = allocTree(t->maxadjnodes);
	//printf("Before copying\n");
	//showNodes(t);	
	nt->maxnodes = t->maxnodes;
	nt->numnodes = t->numnodes;
	nt->maxadjnodes = t->maxadjnodes;
	nt->numadjnodes = t->numadjnodes;

	nt->numintnodes = t->numintnodes;
	nt->numleaves = t->numleaves;

	memcpy(nt->nodes, t->nodes, t->maxnodes*sizeof(int));
	//memcpy(nt->adjnodes, t->adjnodes, t->maxadjnodes*sizeof(int));
	for (i=0; i<t->maxnodes; i++)
		memcpy(nt->children[i], t->children[i], 2*sizeof(int));
	//printf("After copying\n");
	//showNodes(nt);
	//getchar();
	return nt;
}

/*  */
tree* getCopyRelabelled(tree* t) {
	int i;
	int* cnode = (int*)malloc(sizeof(int));

	tree *nt = allocTree(t->maxadjnodes);
	//initTree(nt, t->maxleaves);
	for (i=0; i<nt->maxnodes; i++)
		nt->nodes[i] = NOTHING;
	//for (i=0; i<nt->maxadjnodes; i++)
	//	nt->adjnodes[i] = NOTHING;
	for (i=0; i<nt->maxnodes; i++) {
    nt->children[i][0] = NOTHING;
    nt->children[i][1] = NOTHING;
	}
	nt->maxnodes = t->maxnodes;
	nt->numnodes = t->numnodes;
	nt->numintnodes = t->numintnodes;
	nt->numleaves = t->numleaves;
	nt->maxadjnodes = t->maxadjnodes;
	nt->numadjnodes = t->numadjnodes;
	//printf("Copying tree by relabellation\n");
	//showTree(t);
	*cnode = 0;
	//travCopy(t, nt, 0, cnode, -1);
	travCopy(t, nt, 0, cnode, -1);
	//showTree(nt);
	//getchar();
	return nt;
}

tree* getCopyRelabelled2(tree* t) {
	int i;
	int* cnode = (int*)malloc(sizeof(int));

	tree *nt = allocTree(t->maxadjnodes);
	//initTree(nt, t->maxleaves);
	for (i=0; i<nt->maxnodes; i++)
		nt->nodes[i] = NOTHING;
	//for (i=0; i<nt->maxadjnodes; i++)
	//	nt->adjnodes[i] = NOTHING;
	for (i=0; i<nt->maxnodes; i++) {
    nt->children[i][0] = NOTHING;
    nt->children[i][1] = NOTHING;
	}
	nt->maxnodes = t->maxnodes;
	nt->numnodes = t->numnodes;
	nt->numintnodes = t->numintnodes;
	nt->numleaves = t->numleaves;
	nt->maxadjnodes = t->maxadjnodes;
	nt->numadjnodes = t->numadjnodes;
	//printf("Copying tree by relabellation\n");
	//showTree(t);
	*cnode = 0;
	//travCopy(t, nt, 0, cnode, -1);
	travCopy2(t, nt, 0, cnode, -1);
	//showTree(nt);
	//getchar();
	return nt;
}

int travCopy(tree* tref, tree* tcopy, int node, int *cnt, int father) {

	int tmp = *cnt;
	//printf("travCopy(..., ..., %d, %d, %d)\n", node, *cnt, father);

  (*cnt)++;
	tcopy->numnodes++;
	tcopy->nodes[tmp] = father;
	//showAllNodes(tcopy);
	//getchar();
	if (tref->children[node][0] != -1) {
		//(*cnode)++;
		travCopy(tref, tcopy, tref->children[node][0], cnt, tmp);
		//(*cnode)++;
		//cnode = tmp;
		travCopy(tref, tcopy, tref->children[node][1], cnt, tmp);
		//(*cnode)--;
	}

	return tmp+1;
	
}

int travCopy2(tree* tref, tree* tcopy, int node, int *cnt, int father) {

	int tmp = *cnt;
	//printf("travCopy2(..., ..., %d, %d, %d)\n", node, *cnt, father);

	if (tref->children[node][0] != -1) {
  	(*cnt)++;
		tcopy->numnodes++;
		tcopy->nodes[tmp] = father;
	} else {
		tcopy->numnodes++;
		tcopy->nodes[node] = father;
	}
	//showAllNodes(tcopy);
	//getchar();
	if (tref->children[node][0] != -1) {
		//(*cnode)++;
		travCopy2(tref, tcopy, tref->children[node][0], cnt, tmp);
		//(*cnode)++;
		//cnode = tmp;
		travCopy2(tref, tcopy, tref->children[node][1], cnt, tmp);
		//(*cnode)--;
	}

	return tmp+1;
	
}


void showTree(tree* t) {
	int i;
	for (i=0; i<t->numnodes; i++) {
		printf("%d->%d\n", t->nodes[i], i);
	}	
}

void showOrder(tree* t) {
int i;
	for (i=0; i<t->numadjnodes; i++) {
		//printf("%d ", t->adjnodes[i]);
	}
	printf("\n");	
}	

void showNodes(tree* t) {
  int i;
  for (i=0; i<t->maxnodes; i++) {
    printf("%d, children %d et %d\n", i, t->children[i][0], t->children[i][1]);
  }
}

void showAllNodes(tree* t) {
  int i;
	printf("node\tparent\tchildren (numintnodes=%d, numleaves=%d)\n", t->numintnodes, t->numleaves);
  for (i=0; i<t->maxnodes; i++) {
    printf("%d\t%d\t%d,%d\n", i, t->nodes[i], t->children[i][0], t->children[i][1]);
  }
}

void showNodesInLine(tree* t) {
	int i;
	for (i=0; i<t->maxnodes; i++) {
		printf("%d ", t->nodes[i]);
	}
	//printf("\n");	
}	
	



void duplicateByUneqRecomb(tree* t, int i, int j) {
	int k;
  //int numnode;

	//printf("Before dupli\n");
	//showAllNodes(t);	
	//assert(j>=j);
	//assert(j<t->numadjnodes);
	//printf("Before uneq recomb\n"); showOrder(t);

	for (k=j+1; k<t->numleaves; k++) {
		
		//printf("le fils %d de %d devrait devenir %d\n", t->maxintnodes+k, t->nodes[t->maxintnodes+k], t->maxintnodes+k+j-i+1);
		//printf("si %d == %d\n", t->children[t->nodes[t->maxintnodes+k]][0], t->maxintnodes+k);
		//printf("si %d == %d\n", t->children[t->nodes[t->maxintnodes+k]][1], t->maxintnodes+k);
		if (t->children[t->nodes[t->maxintnodes+k]][0] == t->maxintnodes+k)
			t->children[t->nodes[t->maxintnodes+k]][0] = t->maxintnodes+k+j-i+1;
		else if (t->children[t->nodes[t->maxintnodes+k]][1] == t->maxintnodes+k)
		  t->children[t->nodes[t->maxintnodes+k]][1] = t->maxintnodes+k+j-i+1;
		else { printf("Problme!\n"); exit(0); }
	}

	//recopie les t->nodes des feuilles i  j dans les nodes internes
	memcpy(t->nodes+t->numintnodes, t->nodes+t->maxintnodes+i, (j-i+1)*sizeof(int));

	//maj les children des parents des nouvelles nodes internes
	for (k=i; k<=j; k++) {
		if (t->children[t->nodes[t->maxintnodes+k]][0] == t->maxintnodes+k)
			t->children[t->nodes[t->numintnodes+k-i]][0] = t->numintnodes+k-i;
		else if (t->children[t->nodes[t->maxintnodes+k]][1] == t->maxintnodes+k)
			t->children[t->nodes[t->numintnodes+k-i]][1] = t->numintnodes+k-i;
		else { printf("Problme!\n"); exit(0); }
	}

	for (k=i; k<=j; k++) {
		t->children[t->numintnodes+k-i][0] = t->maxintnodes+k;
		t->children[t->numintnodes+k-i][1] = t->maxintnodes+k+j-i+1;
	}

	for (k=i; k<=j; k++) {
		//printf("t->nodes[%d]=t->nodes[%d]\n", t->numintnodes+k-i, t->maxintnodes+k);
		t->nodes[t->numintnodes+k-i] = t->nodes[t->maxintnodes+k];
		t->nodes[t->maxintnodes+k] = t->numintnodes+k-i;
	}

  t->numintnodes = t->numintnodes + j - i + 1;
		
	memmove(t->nodes+t->maxintnodes+j+1, t->nodes+t->maxintnodes+i, (t->numleaves - i)*sizeof(int));
	
	t->numleaves = t->numleaves + j - i + 1;

	

	/*for (k=i; k<=j; k++) {
	//rznvoie feuilles i  j dans nodes internes
    t->nodes[t->maxintnodes+k]
	for (k=0; k<=(j-i); k++) {
		printf("t->nodes[%d]=t->nodes[%d]\n", t->numintnodes+k, t->maxintnodes+i+k);
		t->nodes[t->numintnodes+k] = t->nodes[t->maxintnodes+i+k];
		t->children[t->numintnodes+k][0] = t->maxintnodes+i+k;
		t->children[t->numintnodes+k][1] = t->maxintnodes+i+k+j-i+1;
		//printf("if %d == %d\n", t->children[t->nodes[t->numintnodes+i+k]][0], t->maxintnodes+k);
		//showAllNodes(t);
		//getchar();
		if (t->children[t->nodes[t->numintnodes+i+k]][0] == t->maxintnodes+k) {
			printf("t->children[%d][0] = %d\n", t->nodes[t->numintnodes+i+k], t->maxintnodes+k);
			t->children[t->nodes[t->numintnodes+i+k]][0] = t->numintnodes+k;
			printf("t->children[%d][1] = %d\n", t->nodes[t->numintnodes+i+k], t->maxintnodes+k+j+1);
			t->children[t->nodes[t->numintnodes+i+k]][1] = t->children[t->nodes[t->numintnodes+i+k]][1]+j-i+1;
		}
		if (t->children[t->nodes[t->numintnodes+i+k]][1] == t->maxintnodes+k)
			 t->children[t->nodes[t->numintnodes+i+k]][1] = t->nodes[t->numintnodes+k];
		
		printf("t->nodes[%d]=%d\n", t->maxintnodes+i+k, t->numintnodes+k);
		t->nodes[t->maxintnodes+i+k] = t->numintnodes+k;
		//ne pas oublier les childrens
		
	} */
	//1t->numintnodes = t->numintnodes + j - i + 1;

	//memcpy(t->nodes+t->numintnodes, t->nodes+, j-i+1*sizeof(int)
	//for (k=i; k<=j; k++) {
	//	t->nodes
		
		
	//memmove(t->nodes+t->maxintnodes+j+1, t->nodes+t->maxintnodes+i, (t->numleaves - i)*sizeof(int));
	
	//t->numleaves = t->numleaves + j - i + 1;
	//memmove(t->adjnodes+j+1, t->adjnodes+i, (t->numadjnodes - i)*sizeof(int));
	//t->numleaves = t->numleaves + 1;
	//t->numadjnodes = t->numadjnodes + j - i + 1;	
	//printf("After uneq recomb\n"); showOrder(t);
	//t->numadjnodes = t->numadjnodes + j - i + 1;
	/*for (k=i; k<=j; k++) {
		//printf("divise node %d\n", t->adjnodes[k]);
		numnode = divide(t, t->adjnodes[k]);
		t->adjnodes[k] = numnode;
		t->adjnodes[k+(j-i+1)] = numnode+1;
		//t->numadjnodes++;
	} */
	//printf("After dupli\n");
	//showAllNodes(t);	
	//getchar();

}

void initDupliTree() {
  dt_root = (dtree*)malloc(sizeof(dtree));
	if (!dt_root) {
		printf("Problme!");
		exit(0);
	}
	dt_root->stree = allocTree(nb_species);
	initTree(dt_root->stree, nb_species);

	minpars = INT_MAX;
}


void buildChildren(dtree* dt, int n) {
	
	int i,j,t;
	int* v;
	tree* tmp, *tmp_t, *treeptr, *prevptr;
  dtree* dt_new;
	static int cnt = 0;
	int p;
	//showNodes(dt->stree);
	//	getchar();
	//if (dt->stree->numadjnodes == nb_species) {
	if (dt->stree->numleaves == nb_species) {
		//showNodes(dt->stree);
		//showTree(dt->stree);
		//getchar();
		p = getParcimony(dt->stree);
		cnt++;
		//tmp = getCopyRelabelled2(dt->stree);
		printf("arbre %d (numleaves=%d)\n", cnt, dt->stree->numleaves);
		/*printf("original:"); showNodesInLine(dt->stree);
		printf("-- copie:"); showNodesInLine(tmp); // pr
		v = getVectorFromTree(tmp);
		//if ((t = searchTree(treestores[dt->stree->numleaves], tmp->nodes+1, tmp->numnodes-1)) != 0)
		if ((t = searchTree(treestores[dt->stree->numleaves], v+1, tmp->numintnodes+tmp->numleaves-1)) != 0)
			printf("Cet arbre existe PAS! (%d)\n", t);
		else
			printf("Cet arbre existe DEJA! (%d)\n", t);  */
		//showAllNodes(dt->stree);
    //printf("APRES\n"); showSuffixTree(treestores[dt->stree->numleaves]);
		//getchar();
		if (p == minpars) {
			tmp_t = getCopyOfTree(dt->stree);
			tmp_t->next = firsttree->next;
			firsttree->next = tmp_t;
		}	
		else
		if (p < minpars) {
			minpars = p;
			printf("p=%d\n", p);
			//delete tous les arbres
			treeptr = firsttree;
			while (treeptr != 0) {
				prevptr = treeptr;
				treeptr = treeptr->next;
				destroy(prevptr);
			}
			firsttree = getCopyOfTree(dt->stree);
			firsttree->next = 0;
			
			//showNodesInLine(dt->stree);
			//getchar();
		}
		//getchar();
	}

	/*for (i=0; i<dt->stree->numadjnodes; i++) {
		for (j=0; j<dt->stree->numadjnodes-i; j++) {
			if (dt->stree->numadjnodes+i < n) { */
	for (i=0; i<dt->stree->numleaves; i++) {
		for (j=0; j<dt->stree->numleaves-i; j++) {
			if (dt->stree->numleaves+i < n) {
			  //cree une nouvelle node
				dt_new = (dtree*)malloc(sizeof(dtree));
				//recopie l'arbre
				dt_new->stree = getCopyOfTree(dt->stree);
				//duplique le ou les sequences
				//printf("dup par ineq recombination de %d a %d\n", j, j+i);
				duplicateByUneqRecomb(dt_new->stree, j, j+i);
				//printf("%d\n", cnt++);
				//printf("Nouvel arbre\n");
				//showTree(dt_new->stree);
				//showOrder(dt_new->stree);
				//explore (construit) la branche si l'arbre cre est nouveau
				
				//getCopyRelabelled(dt_new->stree);

				//arbre nouveau?
				//printf("initial"); showNodesInLine(dt_new->stree);
        //printf("relabel"); showNodesInLine(getCopyRelabelled(dt_new->stree));
				/*tmp = getCopyRelabelled(dt_new->stree);
				if (searchTree(treestores[dt_new->stree->numleaves], tmp->nodes+1, tmp->numnodes-1) != 0)
					printf("Cet arbre existe PAS!\n");
				else
					printf("Cet arbre existe DEJA!\n"); */
				tmp = getCopyRelabelled2(dt_new->stree);
				/*printf("arbre %d (numleaves=%d), ", cnt, dt_new->stree->numleaves);
				printf("original:"); showNodesInLine(dt_new->stree);
				printf("-- copie:"); showNodesInLine(tmp);  */
				v = getVectorFromTree(tmp);
				//if ((t = searchTree(treestores[dt->stree->numleaves], tmp->nodes+1, tmp->numnodes-1)) != 0)
				if ((t = searchTree(treestores[dt_new->stree->numleaves], v+1, tmp->numintnodes+tmp->numleaves-1)) != 0) {
					//printf("Cet arbre existe PAS! (%d)\n", t);
					buildChildren(dt_new, n);
				}

				free(v); destroy(tmp); free(tmp);
				//else
					//printf("Cet arbre existe DEJA! (%d)\n", t);
				//buildChildren(dt_new, n);
				//}
				//else
					//printf("Cet arbre existe DEJA!\n");

				destroy(dt_new->stree);
				free(dt_new->stree);
				free(dt_new);
			}
		}
	}
			
	
}
void loadBootstrapData(char* file, int nbsets) {
	int i, j, k, n, l, cnt=0; //, numset=0;
	char buf[1024];
	char c;
	FILE* fp = fopen(file, "r");
	
	fscanf(fp, "%d %d", &n, &l);
	seq_length = l;
	nb_species = n;
	//printf("n=%d, l=%d\n", n, l);
	fgets(buf, 1024, fp);
	bst_data = (char***)malloc(nbsets*sizeof(char**));
	for (i=0; i<nbsets; i++) {
		bst_data[i] = (char**)calloc(n, sizeof(char*));
		for (j=0; j<n; j++)
			bst_data[i][j] = (char*)calloc(l, sizeof(char));
	}


  //lit le premier
  //probleme des espaces
	for (i=0; i<n; i++) {
		//fgets(buf, 1024, fp);
		j = 0;
		c = getc(fp);
		while (c != '\n') {
			if ((c != ' ') && (j > 10)) {
				strncat(bst_data[0][i], &c, 1);
				if (i == 0) cnt++; //compteur pour savoir ou eon en est de la sequence
			}
			c = getc(fp);
			j++;
		}
		//printf("chaine=%s;\n", bst_data[0][i]);
	}

  while (1) {
		fgets(buf, 1024, fp); //ligne blacnhe
		for (i=0; i<n; i++) {
			j = 0;
			c = getc(fp);
			while ((c != '\n') && (c != -1)) {
				if ((c != ' ') && (j > 10)) {
					strncat(bst_data[0][i], &c, 1);
					if (i == 0) cnt++;
				}
				c = getc(fp);
				j++;

			}
			//printf("chaine=%s;\n", bst_data[0][i]);
		}
		//printf("cnt=%d, l=%d\n", cnt, l); //getchar();
		if (cnt == l)
			break;
	}



	



	//lit les nbsets-1 autres
	fgets(buf, 1024, fp);
	for (i=1; i<nbsets; i++) {
		cnt = 0;	
		//fgets(buf, 1024, fp); //ligne blacnhe
		//printf("%s", buf);
	  //fgets(buf, 1024, fp); //ligne non blacnhe n2
		//printf("%s", buf);
		while (1) {
			
			for (k=0; k<n; k++) {
				j = 0;
				c = getc(fp);
				while ((c != '\n') && (c != -1)) {
					if ((c != ' ') && (j > 10)) {
						strncat(bst_data[i][k], &c, 1);
						if (k == 0) cnt++;
					}
					c = getc(fp);
					j++;

				}
				//if (i == 3) printf("chaine=%s;\n", bst_data[i][k]);
			}
			fgets(buf, 1024, fp);
			//if (i == 3) { printf("cnt=%d, l=%d\n", cnt, l); getchar(); }
			if (cnt == l)
				break;
		}
	}

		data = bst_data[3];
		for (i=0; i<n; i++) {
			printf("%s\n", data[i]);
			//getchar();
		}
		//getchar();
		//printf("chaine=%s;\n", data[i]);
	
}
	

/* charge des donnes d'un fichier au format Phylip */
void loadData(char* file) {
	int i, j, n, l;
	char buf[1024];
	char c;
	FILE* fp = fopen(file, "r");
	
	fscanf(fp, "%d %d", &n, &l);
	seq_length = l;
	nb_species = n;
	printf("n=%d, l=%d\n", n, l);
	fgets(buf, 1024, fp);
	//allocate data
	data = (char**)malloc(n*sizeof(char*));
	for (i=0; i<n; i++)
		data[i] = (char*)malloc(l*sizeof(char));
		
	//probleme des espaces
	for (i=0; i<n; i++) {
		//fgets(buf, 1024, fp);
		j = 0;
		c = getc(fp);
		while (c != '\n') {
			if ((c != ' ') && (j > 10))
				strncat(data[i], &c, 1);
			c = getc(fp);
			j++;
		}
		printf("chaine=%s;\n", data[i]);
	}
	
	while (!feof(fp)) {
		fgets(buf, 1024, fp); //ligne blacnhe
		for (i=0; i<n; i++) {
			j = 0;
			c = getc(fp);
			while ((c != '\n') && (c != -1)) {
				if ((c != ' ') && (j > 10))
					strncat(data[i], &c, 1);
				c = getc(fp);
				j++;
			}
			printf("chaine=%s;\n", data[i]);
		}
	}
	
		
}

	
void DFTraverse(tree* t, int numnode) {
  if (t->children[numnode][0] == -1)
    printf("node %d est une feuille\n", numnode);
  else {
    printf("node %d est une node interne\n", numnode);
    DFTraverse(t, t->children[numnode][0]);
    DFTraverse(t, t->children[numnode][1]);
  }
}

void saveTree(tree* t, int numnode, char* str) {
	char tmp[10];
	if (t->children[numnode][0] == -1) {
	  sprintf(tmp, "%d", numnode);
		strcat(str, tmp);
	}
	else {
    strcat(str, "(");
		saveTree(t, t->children[numnode][0], str);
		strcat(str, ",");
		saveTree(t, t->children[numnode][1], str);
    strcat(str, ")");
	}
}
	
int* getCost(tree* t, int node, int pos) {
	int* cost0;
	int* cost1;
	int* cost2;
	char l;
	int i, j, k, min;
	char* DNA = "ACTG";
	
	cost0 = (int*)malloc(4*sizeof(int));
	//printf("Recherche de la position de la node %d\n", node);
	if (t->children[node][0] == -1) {
		//printf("Recherche de la position de la node %d\n", node);
		//position de node : rech linaire

		//for (i=0; i<t->numadjnodes; i++) {
		//	if (t->adjnodes[i] == node) {
		/*for (i=0; i<t->numleaves; i++) {
			if (t->adjes[i] == node) {
				//i--;
				break;
			}
		}      */
		//showOrder(t);
		//printf("i=%d\n", i); getchar();
		//l = data[i][pos];
    l = data[node - t->maxintnodes][pos];
		//printf("%c\n", l);

		//printf("data=%c\n", l);
		if ((l != 'A') && (l != 'C') && (l != 'T') && (l != 'G') && (l != '-'))
    {
    	showInitialDataset(); getchar();      printf("%c: mauvais caractere! (node=%d, pos=%d (entre %c et %c))", l, node-t->maxintnodes, pos,
    																								data[node - t->maxintnodes][pos-1], data[node - t->maxintnodes][pos+1] );
      exit(0);
    }
		
		if (l == '-')
    {
       cost0[0] = 0;
       cost0[1] = 0;
       cost0[2] = 0;
       cost0[3] = 0;
    }
    else
    {
       for (i=0; i<4; i++)
       {
         if (DNA[i] == l)
           cost0[i] = 0;
         else
           cost0[i] = SHRT_MAX;
                                //cout << cost0[i] << ", ";
       }
		}
	}
	else //pas une feuille
	{
			//printf("getCost de node %d\n", t->children[node][0]);
			cost1 = getCost(t, t->children[node][0], pos);
			//printf("%d %d %d %d\n", cost1[0], cost1[1], cost1[2], cost1[3]);
			//printf("getCost de node %d\n", t->children[node][1]);
			cost2 = getCost(t, t->children[node][1], pos);
			//printf("%d %d %d %d\n", cost2[0], cost2[1], cost2[2], cost2[3]);
			
			//showNodes(t);
			
			for (i=0; i<4; i++)
      {
        cost0[i] = cost1[i] + cost2[i];
      }

      for (i=0; i<4; i++)
      {
      	for (j=0; j<4; j++)
        {
        	if (j != i)
          {
          	if ((min = 1 + cost1[i] + cost2[j]) < cost0[i])
            	cost0[i] = min;
            if ((min = 1 + cost1[j] + cost2[i]) < cost0[i])
              cost0[i] = min;	
          }
     		}
     	}
     	for (i=0; i<4; i++)
      {
        for (j=0; j<4; j++)
        {
          	for (k=0; k<4; k++)
            {
              if ((j != i) && (j != k))
              {
              	if ((min = 2 + cost1[j] + cost2[k]) < cost0[0])
              		cost0[i] = min;
              }
        		}
         	
        }
       }
  //cout << "cost0 " << cost0[0] << ", " << cost0[1] << ", " << cost0[2] << ", " << cost0[3] << endl;
       free(cost1);
       free(cost2);

		}
	//printf("cout de la node %d = (%d %d %d %d)\n", node, cost0[0], cost0[1], cost0[2], cost0[3]);
	return cost0;
}

int getParcimony(tree* t) {
	int* cost;
	int i, pvalue=0, j;
	for (i=0; i<seq_length; i++) {//seq_length; i++) {
	//for (i=40; i<41; i++) {//seq_length; i++) {
		/*for (j=0; j<nb_species; j++) {
			printf("%c\n", data[j][i]);
		}
		printf("\n");*/
		cost = getCost(t, 0, i);
		//printf("%d %d %d %d\n", cost[0], cost[1], cost[2], cost[3]);
		pvalue += getMinimum(cost, 4);
		free(cost);
	}
	return pvalue;
}

int getMinimum(int * vect, int size)
{
	int i, tmp = INT_MAX;

  for (i=0; i<size; i++)
  	if (vect[i] < tmp)
    	tmp = vect[i];
  return tmp;
}

int nbTopol(int n)
{

	int T=0, lim, a;

	printf("nbTopol(%d);\n", n);
	if (n == 1)
		return 0;
	else if (n == 2)
		return 1;
	else {
		if (n%2 == 0)
			lim = n/2;
		else
			lim = (n+1)/2;

		printf("lim = %d\n", lim);
		//getchar();
		printf("for a=1; a<=%d; a++\n", (n-lim));
		for (a=1; a<=(n-lim); a++) {
			
			T += (n-2*a+1) * nbTopol(n - a);
			//getchar();
		}
		/*if (n%2 == 0)
			T += nbTopol(lim);
		else
			T += 2*nbTopol(lim); */
	}
	return T;
}

void reInitSuffixTree(suffixtree* s) {
	int max_int = MAXNUMNODES, i, j;
	for (i=0; i<max_int; i++) {
		//printf("s->maxch=%d\n", s->maxchildren);
		//getchar();
		for (j=0; j<s->maxchildren; j++)
			s->nexts[i][j] = -1;
		s->info[i] = -1;
	}
	s->numnodes = 1;
}


suffixtree* allocSuffixTree(int T) {
  int max_int = MAXNUMNODES, i, j;
	

	suffixtree* s_new = (suffixtree*)malloc(sizeof(suffixtree));
	
	assert(T>0);
	if (s_new == 0) {
			printf("Problme d'allocation mmoire\n");
		exit(0);
	}
		

	s_new->maxchildren = 2*T-1;
	//s_new->prev = (int*)malloc(max_int*sizeof(int));
	//for (i=0; i<max_int; i++)
	//	s_new->prev[i] = -1;
	s_new->info = (int*)malloc(max_int*sizeof(int));
	if (s_new->info == 0) {
		printf("Problme d'allocation mmoire\n");
		exit(0);
	}
	for (i=0; i<max_int; i++)
		s_new->info[i] = -1;
	s_new->nexts = (int**)malloc(max_int*sizeof(int*));
	if (s_new->nexts == 0) {
		printf("Problme d'allocation mmoire\n");
		exit(0);
	}
	for (i=0; i<max_int; i++) {
		s_new->nexts[i] = (int*)malloc(s_new->maxchildren*sizeof(int));
		if (s_new->nexts[i] == 0) {
		printf("Problme d'allocation mmoire\n");
		exit(0);
	}
		for (j=0; j<s_new->maxchildren; j++)
			s_new->nexts[i][j] = -1;
	}
	s_new->numnodes = 1;
	//showSuffixTree_n(s_new, 10);
	return s_new;
}

void showSuffixTree(suffixtree* s) {
	int i,j;
	for (i=0; i<s->numnodes; i++) {
		printf("%d\t%d\t[", i, 0);//s->prev[i]);
		for (j=0; j<s->maxchildren; j++)
			printf("%d ", s->nexts[i][j]);
		printf("]\t%d\n", s->info[i]);
		//printf("]\n");
	}
	//s->prev[0] = -1;
}

void showSuffixTree_n(suffixtree* s, int n) {
	int i,j;
	for (i=0; i<n; i++) {
		printf("%d\t%d\t[", i, 0); //s->prev[i]);
		for (j=0; j<s->maxchildren; j++)
			printf("%d ", s->nexts[i][j]);
		printf("]\n");
	}
}

int* getVectorFromTree(tree* t) {
	int* v = (int*)malloc((t->numintnodes + t->numleaves)*sizeof(int));
	memcpy(v, t->nodes, t->numintnodes*sizeof(int));
	memcpy(v+t->numintnodes, t->nodes+t->maxintnodes, t->numleaves*sizeof(int));
	return v;
}

int searchTree(suffixtree* s, int* data, int d_size) {
	int i,j, tmp;
 	int node = 0;
  //printf("searchTree(.., .., %d)\n", d_size);
	//printf("AVANT\n"); showSuffixTree(s);
	//printf("numnodes=%d\n", s->numnodes);
	/*for (i=0; i<d_size; i++)	
		printf("%d ", data[i]);    */
	//getchar();
	/*printf(" ( ");
	for (i=0; i<d_size-1; i++)
		printf("%d ", data[i]);
	printf(" ) \n");
	getchar();*/
	//for (i=0; i<d_size-1; i++) {
	for (i=0; i<d_size-1; i++) { //modified
		//on parcours l'arbre tant que les donnees correspondent aux noeuds de l'arbre
		//si la donne a la position i est gale a l'info de la node actuelle
		tmp = ((s->nexts[node][data[i]] == -1)?-1:s->info[s->nexts[node][data[i]]]);	
		//printf("if data[i] == s->info[s->nexts[node][data[i]]]\n");
		//printf("if %d == %d ... (i=%d, node=%d, s->nexts[node][data[i]]=%d)\n", data[i], tmp, i, node, s->nexts[node][data[i]]);
    //if ((i == d_size-1) && (s->nexts[node][data[i]] == -1))
		//	return 0;
  	if (data[i] == tmp) {
			//printf("node = %d (s->nexts[%d][%d])\n", s->nexts[node][data[i]], node, data[i]);
			node = s->nexts[node][data[i]]; //passe a la node suivante
		} else {
			//sinon il y a rupture entre l'arbre et les donnes
			//printf("rupture entre l'arbre et les donnes\n");
			for (j=i; j<d_size; j++) {

				//s->prev[0] = -1;
				//s->children[s->numnodes];
				//assert(node > 0);
				//printf("%d ", data[j]);
				//printf("s->prev[%d] = %d\n", s->numnodes, node);
				//s->prev[s->numnodes] = node;
				//printf("s->info[%d] = %d\n", s->numnodes, data[j]);
				s->info[s->numnodes] = data[j];
				//assert(node > 0);
				s->nexts[node][data[j]] = s->numnodes; //pointe sur next node
				node = s->numnodes;
				s->numnodes++;
			}
      //break; //sort de la boucle for
			s->numnodes--;
			return -1;
		}
	}
	return 0;
}

void initTreeStores(int T) {
	int i;

	treestores = (suffixtree_ptr*)malloc(T*sizeof(suffixtree_ptr));
	if (treestores == 0) {
		printf("Problme d'allocation mmoire\n");
		exit(0);
	}
	for (i=1; i<=T; i++)
		treestores[i] = allocSuffixTree(i); //allocate un arbre suffixe de taille i (0..T)
}

void reInitTreeStores(int T) {
	int i;
	for (i=1; i<=T; i++)
		reInitSuffixTree(treestores[i]);
	
}

void destroySuffixTree(suffixtree* s) {
	int i, max_int=MAXNUMNODES;
	for (i=0; i<max_int; i++)
		free(s->nexts[i]);
	free(s->nexts);
	free(s->info);
	//free(s->prev);
}

void destroyTreeStores(int T) {
	int i;
	for (i=2; i<=T; i++)
		destroySuffixTree(treestores[i]);
	//free(treestores);
}
	

void showGoodTrees() {
	char* str;
	tree* treeptr;
	treeptr = firsttree;
	//printf("Min parcimonie = %d\n", minpars);
	
	while (treeptr != 0) {
		str = (char*)calloc(100, sizeof(char));
		
		//printf("before(%d) : %s\n", strlen(str), str);
		saveTree(treeptr, 0, str);
		//printf("after(%d) : %s\n", strlen(str), str);
		//getchar();
		showNodesInLine(treeptr); printf("tree = %s, parci=%d\n", str, minpars);
		treeptr = treeptr->next;
		free(str);
		str = 0;
	}
	
}


void saveTreesToFile(FILE* fp) {
	char* str;
	tree* treeptr;
	//FILE* fp = fopen("trees", "w");
	treeptr = firsttree;
	
	while (treeptr != 0) {
		str = (char*)calloc(100, sizeof(char));
		//str[0] = '\0';
		saveTree(treeptr, 0, str);
		//showNodesInLine(treeptr);
		fprintf(fp, "%s;\n", str);
		treeptr = treeptr->next;
		free(str);
	}
}

char* getBiPartitions(tree* t, int node) {

	char* str = (char*)calloc(100, sizeof(char));
	char*s1, *s2;
  if (t->children[node][0] == -1) {
    //printf("node %d est une feuille\n", numnode);
    sprintf(str, "%d", node);
    return str;
  }
  else {
    //printf("node %d est une node interne\n", numnode);
    s1 = getBiPartitions(t, t->children[node][0]);
    s2 = getBiPartitions(t, t->children[node][1]);
    strcat(s1, s2);
    printf("bipartition : %s\n", s1);
    return s1;
    //union des deux sets
  }
}

short* getBiPartitions2(tree* t, int node) {

	//char* str = (char*)calloc(100, sizeof(char));
	short* bip;
	short* s1, *s2;
	int i, b=0;
  if (t->children[node][0] == -1) {
    //printf("node %d est une feuille\n", numnode);
    bip = (short*)malloc(t->numleaves*sizeof(short));
		memset(bip, 0, t->numleaves*sizeof(short));
		bip[node - t->maxintnodes] = 1;
    return bip;
  }
  else {
    //printf("node %d est une node interne\n", numnode);
    s1 = getBiPartitions2(t, t->children[node][0]);
    s2 = getBiPartitions2(t, t->children[node][1]);
    for (i=0; i<t->numleaves; i++)
    	s1[i] = s1[i] | s2[i];
    //printf("bipartition : %s\n", s1);
		for (i=0; i<t->numleaves; i++) {
    	printf("%d ", s1[i]);
    	b += s1[i]*(int)pow((double)2,(double)(t->numleaves-1-i));
    }
    printf("%d \n", b);
    return s1;
    //union des deux sets
  }
}

short* getBiPartitions3(tree* t, int node, short** bipart) {

	//char* str = (char*)calloc(100, sizeof(char));
	short* bip;
	short* s1, *s2, *s3;
	int i, b=0;
	
	assert(bipart != 0);
  if (t->children[node][0] == -1) {
    bip = (short*)malloc(t->numleaves*sizeof(short));
		memset(bip, 0, t->numleaves*sizeof(short));
		bip[node - t->maxintnodes] = 1;
    return bip;
  }
  else {
		s3 = (short*)malloc(t->numleaves*sizeof(short));
    s1 = getBiPartitions3(t, t->children[node][0], bipart);
    s2 = getBiPartitions3(t, t->children[node][1], bipart);
    for (i=0; i<t->numleaves; i++)
    	s3[i] = s1[i] | s2[i];
		printf("node %d : ", node);
		for (i=0; i<t->numleaves; i++) {
    	printf("%d ", s3[i]);
    	b += s3[i]*(int)pow((double)2,(double)(t->numleaves-1-i));
    }
    printf("%d \n", b);
    //printf("bipart[%d] = \n", node);
    bipart[node] = s3;
    //printf("bipart[%d] = ", node);
		/*for (i=0; i<t->numleaves; i++) {
    	printf("%d ", bipart[node][i]);
    	//b += s1[i]*(int)pow((double)2,(double)(t->numleaves-1-i));
    }
    printf("\n");*/
    return s3;
    //union des deux sets
  }
}

	
void showPartitions() {
	char* str;
	tree* treeptr;
	treeptr = firsttree;
	printf("Min parcimonie = %d\n", minpars);
	while (treeptr != 0) {
		str = (char*)calloc(100, sizeof(char));
		
		//printf("before(%d) : %s\n", strlen(str), str);
		saveTree(treeptr, 0, str);
		
		//printf("after(%d) : %s\n", strlen(str), str);
		//getchar();
		showNodesInLine(treeptr); printf(" %s\n", str);
		getBiPartitions2(treeptr, 0);
		treeptr = treeptr->next;
		free(str);
		str = 0;
	}
}

/***************************************************************************

	void showPartitions2() : 	affiche les partitions renvoyes par
														getBiPartition3 (en paramtre)
	variables globales : firsttree (pointeur sur le premier arbre trouv)

 **************************************************************************/
void showPartitions2() {
	char* str;
	tree* treeptr;
	short** bipart;
	int i, j;
	treeptr = firsttree->next;
	printf("Min parcimonie = %d\n", minpars);
	while (treeptr != 0) {
		str = (char*)calloc(100, sizeof(char));
		saveTree(treeptr, 0, str); //sauve l'arbre dans la chaine str
		showNodesInLine(treeptr); printf(" %s\n", str); //affichages
		bipart = (short**)malloc(treeptr->numintnodes*sizeof(short*));
		getBiPartitions3(treeptr, 0, bipart); //recupere les bipartitions
		for (i=0; i<treeptr->numintnodes; i++) {
			printf("node %d : ", i);
			for (j=0; j<treeptr->numleaves; j++)
				printf("%d ", bipart[i][j]);
			printf("\n");
		}
		free(bipart);
		treeptr = treeptr->next;
		free(str);
		str = 0;
	}
}


void initBipartitions() {
	short** bipart = (short**)malloc(firsttree->numintnodes*sizeof(short*));
	int b=0, i, j, bmax;
	getBiPartitions3(firsttree, 0, bipart); //recupere les bipartitions
	initial_bip_complete = bipart;
			
	//maj var globales de pointage sur les bipartitions initiales
	initial_bip = (int*)malloc(firsttree->numintnodes*sizeof(int*));
	
	bmax = (int)pow((double)2, (double)firsttree->numleaves) / 2;
	
	partition_cnt = (int*)calloc(bmax, sizeof(float));
	
	for (i=0; i<firsttree->numintnodes; i++) {
		b = 0;
		for (j=0; j<firsttree->numleaves; j++) {
    	//printf("%d ", s3[i]);
    	b += initial_bip_complete[i][j]*(int)pow((double)2,(double)(firsttree->numleaves-1-j));
    }
		initial_bip[i] = ((b>=bmax)?bmax*2-1-b:b);
		//peut-etre maj initial_bip_complete en inversant les 0 et 1
	}
}	

void countBipartitions() {

	short** bipart = (short**)malloc(firsttree->numintnodes*sizeof(short*));
	int b=0, i, j, bmax, bmaxdiv2, cnt=0;
	tree* treeptr;
	
	assert(bipart != 0);
	
	
	bmax = (int)pow((double)2, (double)firsttree->numleaves) / 2;
	
	//compte le nombre d'arbres trouvs
	treeptr = firsttree;
	while (treeptr != 0) { cnt++; treeptr = treeptr->next; }
	
	
	
	treeptr = firsttree;
	while (treeptr != 0) {
		getBiPartitions3(treeptr, 0, bipart); //recupere les bipartitions
	
	//bmax = (int)pow((double)2, (double)firsttree->numleaves) / 2;
	//ex bmax=128/2=64 pour 7 especes
	//bmaxdiv2 = bmax/2;
	
		for (i=0; i<firsttree->numintnodes; i++) {
			b = 0;
			for (j=0; j<firsttree->numleaves; j++) {
    		//printf("%d ", s3[i]);
    		//b += initial_bip_complete[i][j]*(int)pow((double)2,(double)(firsttree->numleaves-1-j));
    		b += bipart[i][j]*(int)pow((double)2,(double)(firsttree->numleaves-1-j));
    	}
    	//assert(((b>bmax)?bmax*2-b:b) >= 0);
			//assert(((b>bmax)?bmax*2-b:b) <= 64);
			//printf("partition_cnt[%d]++\n",((b>=bmax)?bmax*2-1-b:b)); getchar();
			partition_cnt[((b>=bmax)?bmax*2-1-b:b)] = partition_cnt[((b>=bmax)?bmax*2-1-b:b)] + 1.0/cnt;
			//peut-etre maj initial_bip_complete en inversant les 0 et 1
		}
		treeptr = treeptr->next;
		printf("\n");
	}	
	
}

void showAllBipartitions() {
	int i, bmax = (int)pow((double)2, (double)nb_species) / 2;
	for (i=0; i<bmax; i++) {
		printf("p=%d, cnt=%3.2f\n", i, partition_cnt[i]);
	}
}

void showGoodPartitions() {
	int i, j;
	printf("Bonnes partitions\n");
	for (j=nb_species-1; j<2*nb_species-1; j++) {
		printf("%d\t", j);
	}
	printf("\n");
	for (i=0; i<nb_species-1; i++) {
		for (j=0; j<nb_species; j++) {
			printf("%s\t", ((initial_bip_complete[i][j])?"*":"."));
		}
		printf("p[%d]=%3.3f\n", initial_bip[i], partition_cnt[initial_bip[i]]);
	}
}


void showDataset(int n) {
	int i;
	for (i=0; i<nb_species; i++) {
		printf("%s\n", bst_data[n][i]);
	}
}		

void showInitialDataset() {
	int i;
	for (i=0; i<nb_species; i++) {
		printf("%s\n", data[i]);
	}
}		
/*************************************************************************

	renracine un arbre par une feuille

 ************************************************************************/
void reroot(tree* t, int node) {

	int lastnode, currnode, tempnode, pnode;
	int tmp;
	
	assert(node >= t->numintnodes);
	
	pnode = t->nodes[node]; //pour apres
	
	lastnode = node;
	currnode = t->nodes[node]; //si 0 stop
	if (currnode != 0) {
		while (currnode != 0) {
			/*printf("BEFORE: currnode=%d (prev=%d, next0=%d, next1=%d), lastnode=%d\n", currnode, t->nodes[currnode],
																																				t->children[currnode][0],
																																				t->children[currnode][1], lastnode);
			*/
			if (t->children[currnode][0] == lastnode)
				t->children[currnode][0] = t->nodes[currnode];
			else if (t->children[currnode][1] == lastnode)
				t->children[currnode][1] = t->nodes[currnode];
			else exit(0);
			tempnode = t->nodes[currnode];
			t->nodes[currnode] = lastnode;
			
			/*printf("AFTER: currnode=%d (prev=%d, next0=%d, next1=%d), lastnode=%d\n", currnode, t->nodes[currnode],
																																				t->children[currnode][0],
																																				t->children[currnode][1], lastnode);
			*/
			lastnode = currnode;
			
			
			currnode = tempnode;
		}
		
		
		if (t->children[lastnode][0] == 0)
			tmp = 0;
		else if (t->children[lastnode][1] == 0)
			tmp = 1;
		else exit(0);
			
		if (t->children[0][0] == lastnode) {
			t->children[lastnode][tmp] = t->children[0][1];
			//t->nodes[t->children[lastnode][tmp]] = lastnode;
		}
		else {
			t->children[lastnode][tmp] = t->children[0][0];
			//t->nodes[t->children[lastnode][tmp]] = lastnode;
		}
		
		t->nodes[t->children[lastnode][tmp]] = lastnode;
			
		t->children[0][0] = node;
		t->children[0][1] = pnode;
		t->nodes[node] = 0;
		t->nodes[pnode] = 0;
		
		
		
	}
}

int areIdentical(tree* t1, tree* t2) {

	tree* t1c = getCopyOfTree(t1);
	tree* t2c = getCopyOfTree(t2);
	
	reroot(t1c, t1c->numintnodes);
	reroot(t2c, t2c->numintnodes);
	
	return compareTrees(t1c, 0, t2c, 0);
}

		
int compareTrees(tree* t1, int n1, tree* t2, int n2) {
	if ((t1->children[n1][0] != -1) &&	(t2->children[n2][0] != -1)) {
		return (
			(compareTrees(t1, t1->children[n1][0], t2, t2->children[n2][0]) &&
			 compareTrees(t1, t1->children[n1][1], t2, t2->children[n2][1])) ||
			(compareTrees(t1, t1->children[n1][0], t2, t2->children[n2][1]) &&
			 compareTrees(t1, t1->children[n1][1], t2, t2->children[n2][0]))
			     );
	}
	else if ((t1->children[n1][0] == -1) &&	(t2->children[n2][0] == -1)) {
		if (n1 == n2)
			return 1;
		else
			return 0;
	}
	else
		return 0;
}
	

/*******************************************************************************

	vire les 	arbres identiques de ceux trouves

 *******************************************************************************/
	
void removeIdenticalTrees() {
	tree* trptr1, *trptr2, *prevptr;
	int i=0, j=0;
	
	trptr1 = firsttree;
	//trptr2 = firsttree;
	//prevptr  = firsttree;
	while (trptr1 != 0) {
		prevptr  = trptr1;
		trptr2 = trptr1->next;//firsttree;
		i = j+1;
		while (trptr2 != 0) {
			//printf("Examine arbres %d et %d\n", trptr1, trptr2);//i, j);
			if (trptr1 != trptr2) {
				//printf("Examine arbres %d et %d\n", j, i);
				if (areIdentical(trptr1, trptr2)) {
					//printf("vire un arbre\n");
					prevptr->next = trptr2->next;
					
					//getchar();
				}	
				else
					prevptr = trptr2;			
			}
			//prevptr = trptr2;
			trptr2 = trptr2->next;
			i++;
		}
		
		trptr1 = trptr1->next;
		j++;
	}
	
	//getchar();
}
	
	
	
	