/***************************************************************************
                          main.c  -  description
                             -------------------
    begin                : Tue Jun 13 2000
    copyright            : (C) 2000 by Olivier ELEMENTO
    email                : olivier@ligm.igh.cnrs.fr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

//#include "pvm3.h"
#include <stdio.h>
#include <stdlib.h>
#include "tree.h"

//#include "memtrace.h"
#include "pvm3.h"

#define SLAVENAME "fastdupli_slave"
int main(int argc, char *argv[])
{
  //general
	//FILE* fp;	
	int i, j, k;
	tree* t;
	tree* t_initial;
	tree* nexttree;
	
	//pvm
	int mytid;
	int tids[32];
	int nproc, numt, nhost, narch, bufid;
  //float result[32];
  struct pvmhostinfo *hostp;

  char* rec_tree;
  int rec_parc, nb_trees, total_nb_trees=0, total_nb_id_trees=0, num_sets, data_share, data_rest;
  int maxnodes, numintnodes, numleaves;
	

  //charge les donnees initiales
  loadData("iglcnogaps.phy");

  //lance le calcul des arbres les plus parcimonieux
	initTreeStores(nb_species);
	initDupliTree();
	initTree(dt_root->stree, nb_species);
	buildChildren(dt_root, nb_species);
	
	//firstree est un pointeur vers le premier arbre
	t_initial = getCopyOfTree(firsttree);
	
	//choisit les bipartitions a garder
	initBipartitions();
	
	//charge les donnes de bootstrap
  loadBootstrapData("iglcnogaps.bst", 100);


  /* enroll in pvm */
  mytid = pvm_mytid();
	/* Set number of slaves to start */
  pvm_config( &nhost, &narch, &hostp );

  nproc = nhost;
	printf("start up slave tasks\n");
  numt=pvm_spawn(SLAVENAME, (char**)0, 0, "", nproc, tids);
  if( numt < nproc ){
       printf("\n Trouble spawning slaves. Aborting. Error codes are:\n");
       for( i=numt ; i<nproc ; i++ ) {
          printf("TID %d %d\n",i,tids[i]);
       }
       for( i=0 ; i<numt ; i++ ){
          pvm_kill( tids[i] );
       }
       pvm_exit();
       exit(1);
  }
  printf("SUCCESSFUL\n");

  data_share = 100 / nproc;
  data_rest = 100 % nproc;

 	
	for (i=0; i<nproc; i++) {
		bufid = pvm_mkbuf(PvmDataDefault);
		pvm_setsbuf(bufid);
		pvm_pkint(&nb_species, 1, 1);
		pvm_pkint(&seq_length, 1, 1);
		num_sets = (i==nproc-1)?data_share+data_rest:data_share;
		pvm_pkint(&num_sets, 1, 1);
		//pvm_pkstr(sent_data);
		for (k=i*data_share; k<i*data_share+num_sets; k++) {
			data = bst_data[k];
			for (j=0; j<nb_species; j++) {
				printf("sending %s to task %d\n", data[j], tids[i]);
				pvm_pkstr(data[j]);
			}
		}
		//printf("sending %s to task %d\n", data[0], tids[i]);
		pvm_send(tids[i], 0);
		pvm_freebuf(bufid);
	}	
	
	printf("Start receiving data\n");
	for (i=0; i<100; i++) {
		pvm_recv(-1, 5);
		//printf("received!");
		pvm_upkint(&rec_parc, 1, 1);
		minpars = rec_parc;
		
		//recois le nombre d'arbres pour ce pseudosample
		pvm_upkint(&nb_trees, 1, 1);
		//maj le nb total d'arbres
		total_nb_trees += nb_trees;
		
		
		pvm_upkint(&maxnodes, 1, 1);
		
		//recup le nomdres de noeuds internes
  	pvm_upkint(&numintnodes, 1, 1);
  	
  	//recup le nombre de feuilles
  	pvm_upkint(&numleaves, 1, 1);
		
		//recoit les arbres
		for (j=0; j<nb_trees; j++) {
		
			//la forme Phylip
			rec_tree = (char*)calloc(100, sizeof(char));
			pvm_upkstr(rec_tree);
			//printf("tree = %s, parci=%d\n", rec_tree, rec_parc);
			
			//cree un nouvel arbre
			t = allocTree(nb_species);
			
			//recupere les prec
			pvm_upkint(t->nodes, maxnodes, 1);
			
			//recupere les children
			for (k=0; k<maxnodes; k++) {
				pvm_upkint(t->children[k], 2, 1);
			}
			
			t->maxnodes = maxnodes;
			t->numintnodes = numintnodes;
			t->numleaves = numleaves;
			
			//printf("stocke les arbres\n");
			if (j == 0) {
				firsttree = t;
				firsttree->next = 0;
			} else {
				t->next = firsttree;
				firsttree = t;
			
			}
			
			
			//showAllNodes(t);
		}
		printf("TREES\n");	
		showGoodTrees();
			
		//printf("vire les arbrss identiques\n");
		removeIdenticalTrees();
		
		
		printf("GOOD TREES\n");	
		showGoodTrees();	
		//showGoodTrees();
		printf("\n\n");	
		//printf("compte les bipartitions des arbres restants\n");
		countBipartitions();
		printf("\n");	
			
		nexttree = firsttree;
		while (nexttree != firsttree) {
			if (areIdentical(t_initial, nexttree))
				total_nb_id_trees++;
			nexttree = nexttree->next;
		}
		
		
		//printf("vire les arbres\n");
			
			
	
	}
	storeResults("results.txt", (double)total_nb_id_trees/total_nb_trees);
	showAllBipartitions();
	showGoodPartitions();
	
	/* Program Finished exit PVM before stopping */
  pvm_exit();
	

  return EXIT_SUCCESS;
}
