/***************************************************************************
                          tree.h  -  description
                             -------------------
    begin                : Tue Jun 13 2000
    copyright            : (C) 2000 by Olivier ELEMENTO
    email                : olivier@ligm.igh.cnrs.fr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <stdlib.h>
#include <stdio.h>

#define NOTHING -1

typedef struct _tree {

	short degree;
	
	int maxnodes;
	int numnodes;

	int numintnodes;
	int numleaves;

	int maxintnodes;
	int maxleaves;
	int* nodes;
	
	//nodes fils (navigation, traverse)
  int** children;

	int maxadjnodes;
	int numadjnodes;
	int* adjnodes;
	struct _tree* next;
} tree;

tree* firsttree; //stockage des arbres (liste chaine)

typedef struct _duplitree {
	tree* stree;                   //arbre de sequence
	struct _duplitree* children;   //fils
} dtree;

dtree* dt_root;
char** data;

char*** bst_data;

float* partition_cnt; //contient le nombre d'occurences de chaque bipart
int* initial_bip; //pointe sur les bipartitions initiales
short** initial_bip_complete;

int nb_species;
int seq_length;
int minpars;

typedef struct _suffixtree {
	int numnodes;
	int maxchildren;

	//int* prev;   //node precedents
	int** nexts; //nodes suivantes
	//int** info;  //info
	int* info;

	//struct _suffixtree* next;
} suffixtree;

typedef suffixtree* suffixtree_ptr;
	
suffixtree** treestores; //stockent les arbres

suffixtree* allocSuffixTree(int T);
void initTreeStores(int T);
void reInitTreeStores(int T);
void destroyTreeStores(int T);
tree* allocTree(int T);

void duplicateByUneqRecomb(tree* t, int i, int j);
void initTree(tree* t, int T);
void showTree(tree* t);
void showOrder(tree* t);
void showNodes(tree* t);
void showAllNodes(tree* t);
void showNodesInLine(tree* t);
void showGoodTrees();
int divide(tree* t, int numnode);
void DFTraverse(tree* t, int numnode);

void initDupliTree();
void buildChildren(dtree* dt, int n);


void loadData(char* file);

int* getCost(tree* t, int node, int pos);
int getParcimony(tree* t);
int getMinimum(int * vect, int size);

tree* getCopyRelabelled(tree* t);
tree* getCopyRelabelled2(tree* t);
tree* getCopyOfTree(tree* t);
int travCopy(tree* tref, tree* tcopy, int node, int *cnt, int father);
int travCopy2(tree* tref, tree* tcopy, int node, int *cnt, int father);

int nbTopol(int n);
int* getVectorFromTree(tree* t);

int searchTree(suffixtree* s, int* data, int d_size);
void showSuffixTree(suffixtree* s);
void showSuffixTree_n(suffixtree* s, int n);


void loadBootstrapData(char* file, int nbsets);
void saveTreesToFile(FILE* fp);

void reroot(tree* t, int node); //reenracine un arbre par une node (une feuille)
int areIdentical(tree* t1, tree* t2); //compare 2 arbres
int compareTrees(tree* t1, int n1, tree* t2, int n2);


void initBipartitions();
void showGoodPartitions();
void showAllBipartitions();
void countBipartitions();
void removeIdenticalTrees();

void showInitialDataset();

